function [timePoints, singleCellProfiles, parameterSets] = SimulateLPS(numCells)
% Simulates single-cell LPS-dependent NF-kB responses.  This program
% requires the data files:
%   parameterMeans.txt
%   parameterDeviations.txt
%   parameterRelGaussianWeight.txt
% which correspond to the fitted distributions for the parameters:
%   IKK0 tr3 tr1 tr2 tp1 a7 r4 k02 r1 a4 IkBa_NFkB0
% Outputs:
%   timePoints - timepoints associated with data in singleCellProfiles
%   singleCellProfiles - scaled nf-kb response, where each cell is arranged
%       a row
%   parameterSets - the corresponding parameter set used to generate the
%       profile

%% Initialize storage
timePoints = 0:3:240;
singleCellProfiles = zeros([numCells length(timePoints)]);
parameterSets = zeros([numCells 11]);

%% Simulation parameters for LPS
% order of LPS specific parameters (same as Covert 2005)
% kL2M kMdeg tauM kL2T kTdeg tauT trifLag
lpsFull = [0.075 0.093 20 0.42 0.46 20 50];  %parameters for model with
    % MyD88 and Trif pathways intact
lpsNoTrif = [0.075 0.093 20 0 0.46 20 50]; %parameters for model without
    % Trif pathway
pTrif = 0.55; %probability of activating the Trif pathway

%% Load in fitted parameter distributions
% parameter distributions are fit according to two-gaussian distributions
% order of parameters in files
% IKK0 tr3 tr1 tr2 tp1 a7 r4 k02 r1 a4 IkBa_NFkB0
% The two means and deviations in each row are for the two gaussians used
% to fit the parameter distributions.  The RelGaussianWeight is the
% fraction in Table S1.
parameterMeans = dlmread('parameterMeans.txt');
parameterDeviations = dlmread('parameterDeviations.txt');
parameterRelGaussianWeight = dlmread('parameterRelGaussianWeight.txt');

%% Run the simulations
for simulation = 1:numCells
    % pick parameters from distributions
    for parameter = 1:11
        % distributions are fit as two-gaussians, select gaussian and
        % generate parameter
        randNum = rand();
        if randNum < parameterRelGaussianWeight(parameter,1);
            parameterSets(simulation,parameter) = random('normal',...
                parameterMeans(parameter,1),...
                parameterDeviations(parameter,1));
        else
            parameterSets(simulation,parameter) = random('normal',...
                parameterMeans(parameter,2),...
                parameterDeviations(parameter,2));
        end
    end
    % decide of Trif pathway is active
    isTrif = rand()<pTrif;
    % simulate nfkb profile
    if isTrif
        nfkbLPS = lps_nfkbn(parameterSets(simulation,:),timePoints,lpsFull);
    else
        nfkbLPS = lps_nfkbn(parameterSets(simulation,:),timePoints,lpsNoTrif);
    end
    % normalize trace to min/max of 0/1
    scaled_nfkbLPS = (nfkbLPS - min(nfkbLPS))/(max(nfkbLPS)-min(nfkbLPS));
    singleCellProfiles(simulation,:) = scaled_nfkbLPS;
end

function nfkbn_baseline = lps_nfkbn(ks, times, lps_ks)

% Original parameter next to inputted parameter
LPS = ks(1) ; %0.1
tr3 = ks(2); % 0.0168
tr1 = ks(3); % 0.2448
tr2 = ks(4); % 0.99
tp1 = ks(5); % 0.018
a7 =  ks(6); % 11.1
r4  = ks(7); % 1.221
k02 = ks(8); % 0.0072 
r1  = ks(9); % 0.2442
a4 = ks(10); % 30 ;
IkBa_NFkB0 = ks(11); % 0.0825639   

% Constants for Covert 2005 additions
kL2M =    lps_ks(1); 
kMdeg =   lps_ks(2); 
tauM =    lps_ks(3); 
kL2T =    lps_ks(4);
kTdeg =   lps_ks(5);
tauT =    lps_ks(6);
trifLag = lps_ks(7);

% Here are all the default constants for Hoffmann's model
% Original parameters changed in current model are commented out.
a1 = 1.35 ;% 1/(uM*min) Association between IKK and IkBa (No NFkB)
a2 = 0.36 ;% 1/(uM*min) Association between IKK and IkBb (No NFkB)
a3 = 0.54 ;% 1/(uM*min) Association between IKK and IkBe (No NFkB)
% a4 = 30 ;% 1/(uM*min) Association between IkBa and NFkB
a5 = 30 ;% 1/(uM*min) Association between IkBb and NFkB
a6 = 30 ;% 1/(uM*min) Association between IkBe and NFkB
% a7 = 11.1 ;% 1/(uM*min) Association between IKK and IkBa-NFkB
a8 = 2.88 ;% 1/(uM*min) Association between IKK and IkBb-NFkB
a9 = 4.2 ;% 1/(uM*min) Association between IKK and IkBe-NFkB
d1 = 0.075 ;% 1/min Disassociation between IKK and IkBa (No NFkB)
d2 = 0.105 ;% 1/min Disassociation between IKK and IkBb (No NFkB)
d3 = 0.105 ;% 1/min Disassociation between IKK and IkBe (No NFkB)
d4 = 0.03 ;% 1/min Disassociation between IkBa and NFkB
d5 = 0.03 ;% 1/min Disassociation between IkBb and NFkB
d6 = 0.03 ;% 1/min Disassociation between IkBe and NFkB
deg1 = 0.00675 ;% 1/min Rate of constitute degradation of free IkB
deg4 = 0.00135 ;% 1/min Rate of constitute degradation of IkB-NFkB
k01 = 0.0048 ;%  Transport rate of nuclear NFkB to cytoplasm
% k02 = 0.0072 ;%
k1 = 5.4 ;%  Transport rate of cytoplasmic NFkB to nucleus
k2 = 0.82944 ;% 1/min Transport rate of IkB-NFkB to the nucleus
% r1 = 0.2442 ;% 1/min Reaction rate between IKK and IkBa (No NFkB)
r2 = 0.09 ;% 1/min Reaction rate between IKK and IkBb (No NFkB)
r3 = 0.132 ;% 1/min Reaction rate between IKK and IkBe (No NFkB)
% r4 = 1.221 ;% 1/min Reaction rate between IKK and IkBa-NFkB
r5 = 0.45 ;% 1/min Reaction rate between IKK and IkBb-NFkB
r6 = 0.66 ;% 1/min Reaction rate between IKK and IkBe-NFkB
% tp1 = 0.018 ;% 1/min Rate of IkB nuclear import
tp2 = 0.012 ;% 1/min Rate of IkB nuclear export
% tr1 = 0.2448 ;%  IkB isoform mRNA transcript translation rate
% tr2 = 0.99 ;%  Enhancement rate of NFkB on IkBa transcription
tr2a = 9.21375E-05 ;%  Constitutive rate of IkBa transcription
tr2b = 0.000010701 ;%  Constitutive rate of IkBb transcription
tr2e = 0.000007644 ;%  Constitutive rate of IkBe transcription
% tr3 = 0.0168 ;% 1/min Degradation rate of IkB transcripts
defaultConstants = [a1; a2; a3; a4; a5; a6; a7; a8; a9; d1; d2; d3; d4; d5; d6; deg1; deg4; k01; k02; k1; k2; r1; r2; r3; r4; r5; r6; tp1; tp2; tr1; tr2; tr2a; tr2b; tr2e; tr3; LPS; kL2M; kMdeg; tauM; kL2T; kTdeg; tauT; trifLag];

% Here are the initial conditions
IkBa0 = 0.185339 ;%  
% IkBa_NFkB0 = 0.0825639 ;%  
IkBan0 = 0.18225 ;%  
IkBan_NFkBn0 = 0.00140031 ;%  
IkBat0 = 0.0054872 ;%  
IkBb0 = 0.0212451 ;%  
IkBb_NFkB0 = 0.00934616 ;%  
IkBbn0 = 0.021088 ;%  
IkBbn_NFkBn0 = 0.000313095 ;%  
IkBbt0 = 0.000636964 ;%  
IkBe0 = 0.0151759 ;%  
IkBe_NFkB0 = 0.00667621 ;%  
IkBen0 = 0.0150637 ;%  
IkBen_NFkBn0 = 0.000223652 ;%  
IkBet0 = 0.000455 ;%  
IKK0 = 0 ; %  
IKK_IkBa0 = 0 ;%  
IKK_IkBa_NFkB0 = 0 ;%  
IKK_IkBb0 = 0 ;%  
IKK_IkBb_NFkB0 = 0 ;%  
IKK_IkBe0 = 0 ;%  
IKK_IkBe_NFkB0 = 0 ;%  
NFkB0 = 0.000256516 ;%  
NFkBn0 = 0.000220118 ;%  
fr0 = 0.5 ;%  
MyD880 = 0;
Trif0 = 0; 
dTrif0 = 0;

initial_conditions = [IkBa0; IkBa_NFkB0; IkBan0; IkBan_NFkBn0; IkBat0; IkBb0; IkBb_NFkB0; IkBbn0; IkBbn_NFkBn0; IkBbt0; IkBe0; IkBe_NFkB0; IkBen0; IkBen_NFkBn0; IkBet0; IKK0; IKK_IkBa0; IKK_IkBa_NFkB0; IKK_IkBb0; IKK_IkBb_NFkB0; IKK_IkBe0; IKK_IkBe_NFkB0; NFkB0; NFkBn0; fr0; MyD880; Trif0; dTrif0];


options = odeset('RelTol',1e-4);
sol=dde23(@lps_NFkB_eq_set, trifLag, initial_conditions, times, options,defaultConstants);
y = deval(sol,0:3:240)';
nfkbn_baseline = y(:,24);



function dy = lps_NFkB_eq_set(t,y,Z,c)
% this is going to contain all of the equations for the 
% Covert model of NF-kB response to LPS (a modified Hoffmann/Levchenko model).

% put the constants back in their names
a1 = c(1);  a2 = c(2);  a3 = c(3);  a4 = c(4);  a5 = c(5);
a6 = c(6);  a7 = c(7);  a8 = c(8);  a9 = c(9);  d1 = c(10);
d2 = c(11); d3 = c(12); d4 = c(13); d5 = c(14); d6 = c(15);
deg1 = c(16); deg4 = c(17); k01 = c(18); k02 = c(19); k1 = c(20);
k2 = c(21); r1 = c(22); r2 = c(23); r3 = c(24); r4 = c(25); 
r5 = c(26); r6 = c(27); tp1 = c(28); tp2 = c(29); tr1 = c(30);
tr2 = c(31); tr2a = c(32); tr2b = c(33); tr2e = c(34); tr3 = c(35);
LPS = c(36); kL2M = c(37); kMdeg = c(38); tauM = c(39); kL2T = c(40);
kTdeg = c(41); tauT = c(42); trifDelayTime = c(43);

% get the lag variable for the Trif pathway -- only one lag, so 1st column
trifLag = Z(:,1);

% The y array is as follows:
% y(1) = IkBa           y(2) = IkBa_NFkB           y(3) = IkBan
% y(4) = IkBan_NFkBn    y(5) = IkBat               y(6) = IkBb
% y(7) = IkBb_NFkB      y(8) = IkBbn               y(9) = IkBbn_NFkBn 
% y(10)= IkBbt          y(11)= IkBe                y(12)= IkBe_NFkB
% y(13)= IkBen          y(14)= IkBen_NFkBn         y(15)= IkBet 
% y(16)= IKK            y(17)= IKK_IkBa            y(18)= IKK_IkBa_NFkB 
% y(19)= IKK_IkBb       y(20)= IKK_IkBb_NFkB       y(21)= IKK_IkBe 
% y(22)= IKK_IkBe_NFkB  y(23)= NFkB                y(24)= NFkBn 
% y(25)= fr             y(26)= MyD88               y(27)= Trif
% y(28)= Trif(+delay)

dy(23) = -a4*y(1)*y(23)+d4*y(2)-a5*y(6)*y(23) +d5*y(7) -a6*y(11) *y(23) +d6*y(12) -a4*y(17) *y(23) +(r4+d4)*y(18) -a5*y(19) *y(23) +(r5+d5)*y(20) -a6*y(21) *y(23) +(r6+d6)*y(22) -k1*y(23) +k01*y(24) +deg4*y(2) +deg4*y(7) +deg4*y(12) ;
dy(24) = k1*y(23) -a4*y(3) *y(24) +d4*y(4) -a5*y(8) *y(24) +d5*y(9) -a6*y(13) *y(24) +d6*y(14) -k01*y(24) ;
dy(1) = -a1*y(16) *y(1) +d1*y(17) -a4*y(1) *y(23) +d4*y(2) +tr1*y(5) -deg1*y(1) -tp1*y(1) +tp2*y(3) ;
dy(6) = -a2*y(16) *y(6) +d2*y(19) -a5*y(6) *y(23) +d5*y(7) +tr1*y(10) -deg1*y(6) - tp1*y(6) +tp2*y(8) ;
dy(11) = -a3*y(16) *y(11) +d3*y(21) -a6*y(11) *y(23) +d6*y(12) +tr1*y(15) - deg1*y(11) -tp1*y(11) +tp2*y(13) ;
dy(5)  = tr2a+tr2*y(24) ^2-tr3*y(5) ;
dy(3)  = tp1*y(1) -tp2*y(3) -a4*y(3) *y(24) +d4*y(4) ;
dy(4) = a4*y(3) *y(24) -d4*y(4) -k2*y(4) ;
dy(10)  = tr2b-tr3*y(10) ;
dy(8)  =  tp1*y(6) -tp2*y(8) -a5*y(8) *y(24) +d5*y(9) ;
dy(9)  = a5*y(8) *y(24) -d5*y(9) - 0.5 * y(25) *k2*y(9) ;
dy(15)  = tr2e-tr3*y(15) ;
dy(13) = tp1*y(11) -tp2*y(13) -a6*y(13) *y(24) +d6*y(14) ;
dy(14) = a6*y(13) *y(24) -d6*y(14) - 0.5 * k2*y(14) ;

% departures from the Hoffmann model
dy(26) = (kL2M*LPS - kMdeg*y(26))/tauM;
dy(27) = (kL2T*LPS - kTdeg*y(27))/tauT;
if(t<= trifDelayTime)
    dy(28) = 0;
else
    dy(28) = (kL2T*LPS - kTdeg*trifLag(27,1))/tauT;
end;



dy(16) = dy(26)+dy(28)-k02*y(16) -a1*y(16) *y(1) +(d1+r1)*y(17) -a2*y(16) *y(6) +(d2+r2)*y(19) -a3*y(16) *y(11) +(d3+r3)*y(21) -a7*y(16) *y(2) +(d1+r4)*y(18) -a8*y(16) *y(7) +(d2+r5)*y(20) -a9*y(16) *y(12) +(d3+r6)*y(22) ;

dy(17) = a1*y(16) *y(1) -(d1+r1)*y(17) -a4*y(17) *y(23) +d4*y(18) ;
dy(19) = a2*y(16) *y(6) -(d2+r2)*y(19) -a5*y(19) *y(23) +d5*y(20) ;
dy(21) = a3*y(16) *y(11) -(d3+r3)*y(21) -a6*y(21) *y(23) +d6*y(22) ;
dy(2)  = a4*y(1) *y(23) -d4*y(2) -a7*y(16) *y(2) +d1*y(18) +k2*y(4) -deg4*y(2) ;
dy(7) = a5*y(6) *y(23) -d5*y(7) -a8*y(16) *y(7) +d2*y(20) + 0.5 * y(25) *k2*y(9) -deg4*y(7) ;
dy(12) = a6*y(11) *y(23) -d6*y(12) -a9*y(16) *y(12) +d3*y(22) + 0.5 * k2*y(14) -deg4*y(12) ;
dy(18) = a7*y(16) *y(2) +a4*y(17) *y(23) -(d1+d4+r4)*y(18) ;
dy(20) = a8*y(16) *y(7) +a5*y(19) *y(23) -(d2+d5+r5)*y(20) ;
dy(22) = a9*y(16) *y(12) +a6*y(21) *y(23) -(d3+d6+r6)*y(22) ;
dy(25) = -.5/(1 + t)^2;

dy = [dy(1);dy(2);dy(3);dy(4);dy(5);dy(6);dy(7);dy(8);dy(9);dy(10);dy(11);dy(12);dy(13);dy(14);dy(15);dy(16);dy(17);dy(18);dy(19);dy(20);dy(21);dy(22);dy(23);dy(24);dy(25); dy(26);dy(27); dy(28)];




    